library jaguar_auth.authoriser;

import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_auth/src/entity/entity.dart';
import 'package:jaguar_session/jaguar_session.dart';

class WrapUserAuthoriser extends RouteWrapper<UserAuthoriser> {
  final AuthModelManager modelManager;

  final String authorizationIdKey;

  final String id;

  const WrapUserAuthoriser(this.modelManager,
      {this.authorizationIdKey: 'id', this.id});

  UserAuthoriser createInterceptor() =>
      new UserAuthoriser(modelManager, authorizationIdKey: authorizationIdKey);
}

class UserAuthoriser extends Interceptor {
  final AuthModelManager modelManager;

  final String authorizationIdKey;

  UserAuthoriser(this.modelManager, {this.authorizationIdKey: 'id'});

  @Input(SessionInterceptor)
  Future<UserModel> pre(Request req, SessionManager sessionManager) async {
    await sessionManager.validate();

    String id = sessionManager.getInValue(authorizationIdKey);
    if (id is! String || id.isEmpty) {
      throw new UnAuthorizedError();
    }

    UserModel subject = await modelManager.fetchModelByAuthorizationId(id);

    if (subject == null) {
      throw new UnAuthorizedError();
    }

    return subject;
  }

  @InputRouteResponse()
  @Input(SessionInterceptor)
  Future<Response> post(
      Request req, Response resp, SessionManager sessionManager) async {
    return await sessionManager.updateResponse(req, resp);
  }
}
