library jaguar_auth.authenticator;

import 'dart:async';
import 'package:jaguar_auth/src/hasher/hasher.dart';

abstract class UserModel {
  String get authenticationId;

  String get authenticationKeyword;

  String get authorizationId;
}

/// Checks password for given username and returns the user if passwords match
abstract class AuthModelManager {
  Future<UserModel> fetchModelByAuthenticationId(String authenticationId);

  Future<UserModel> fetchModelByAuthorizationId(String authorizationId);

  Future<UserModel> authenticate(String authId, String keyword);
}

/// Checks password for given username and returns the user if passwords match
class WhiteListPasswordChecker implements AuthModelManager {
  final Map<String, UserModel> models;

  final Hasher hasher;

  const WhiteListPasswordChecker(Map<String, UserModel> models, {Hasher hasher})
      : models = models ?? const {},
        hasher = hasher ?? const NoHasher();

  Future<UserModel> authenticate(String username, String password) async {
    UserModel model = await fetchModelByAuthenticationId(username);

    if (model == null) {
      return null;
    }

    if (!hasher.verify(password, model.authenticationKeyword)) {
      return null;
    }

    return model;
  }

  Future<UserModel> fetchModelByAuthenticationId(String authName) async =>
      models.values.firstWhere((model) => model.authenticationId == authName,
          orElse: () => null);

  Future<UserModel> fetchModelByAuthorizationId(String sessionId) async {
    if (!models.containsKey(sessionId)) {
      return null;
    }

    return models[sessionId];
  }
}
